"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = tslib_1.__importStar(require("path"));
const playwright_1 = require("playwright");
const simple_fake_server_1 = require("simple-fake-server");
const SUT = tslib_1.__importStar(require("../index"));
const utils_1 = require("../../../../utils");
describe('record requests to', () => {
    let browser = undefined;
    let fakeServer = undefined;
    beforeAll(() => {
        fakeServer = new simple_fake_server_1.FakeServer(1239);
        fakeServer.start();
        //The FakeServer now listens on http://localhost:1239
    });
    afterAll(() => {
        if (fakeServer) {
            fakeServer.stop();
        }
    });
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    beforeEach(() => { });
    afterEach(async () => {
        if (browser) {
            await browser.close();
        }
    });
    test('should record successufull requests', async () => {
        // Given
        browser = await playwright_1.chromium.launch({ headless: true });
        const context = await browser.newContext({ viewport: null });
        const page = await context.newPage();
        const responseBody = {
            prop1: 'foobar',
        };
        const responseHeaders = {
            'foo-header': 'bar',
        };
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/foobar')
                .willReturn(responseBody, 200, responseHeaders);
        const requests = [];
        const callback = (request) => requests.push(request);
        const takeAllPredicate = () => false;
        // When
        await SUT.recordRequestsTo('/foobar', takeAllPredicate, page, callback);
        await page.goto(`file:${path.join(__dirname, 'record-requests-to.test.html')}`);
        await (0, utils_1.sleep)(3000);
        // Then
        expect(requests.length).toBe(1);
        const stringifiedRequest = await (0, utils_1.stringifyRequest)(requests[0]);
        const request = JSON.parse(stringifiedRequest);
        expect(request.url).toContain('?foo=bar');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(request.response.status).toBe(200);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(await request.response.payload).toMatchObject(responseBody);
    });
    test('should record failed requests 500', async () => {
        // Given
        browser = await playwright_1.chromium.launch({ headless: true });
        const context = await browser.newContext({ viewport: null });
        const page = await context.newPage();
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/500')
                .willFail(500);
        const requests = [];
        const callback = (request) => requests.push(request);
        const takeAllPredicate = () => false;
        // When
        await SUT.recordRequestsTo('/500', takeAllPredicate, page, callback);
        await page.goto(`file:${path.join(__dirname, 'record-failed-requests-500.test.html')}`);
        await (0, utils_1.sleep)(3000);
        // Then
        expect(requests.length).toBe(1);
        const stringifiedRequest = await (0, utils_1.stringifyRequest)(requests[0]);
        const request = JSON.parse(stringifiedRequest);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(request.response.status).toBe(500);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(request.response.statusText).toBe('Internal Server Error');
    });
    test('should encode HTML response', async () => {
        // Given
        browser = await playwright_1.chromium.launch({ headless: true });
        const context = await browser.newContext({ viewport: null });
        const page = await context.newPage();
        const requests = [];
        const callback = (request) => requests.push(request);
        const takeAllPredicate = () => false;
        // When
        await SUT.recordRequestsTo('/', takeAllPredicate, page, callback);
        await page.goto(`file:${path.join(__dirname, 'record-requests-to.test.html')}`);
        await (0, utils_1.sleep)(3000);
        // Then
        const htmlPageRequest = requests[0];
        const stringifiedRequest = await (0, utils_1.stringifyRequest)(htmlPageRequest);
        const request = JSON.parse(stringifiedRequest);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(request.response.payload).not.toContain('<script>');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(request.response.payload).toContain('&lt;script&gt;');
    });
    test('should record postdata on failed POST requests 502', async () => {
        // Given
        browser = await playwright_1.chromium.launch({ headless: true });
        const context = await browser.newContext({ viewport: null });
        const page = await context.newPage();
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .post()
                .to('/502')
                .willFail(502);
        const requests = [];
        const callback = (request) => requests.push(request);
        const takeAllPredicate = () => false;
        // When
        await SUT.recordRequestsTo('/502', takeAllPredicate, page, callback);
        await page.goto(`file:${path.join(__dirname, 'record-failed-requests-502.test.html')}`);
        await (0, utils_1.sleep)(3000);
        // Then
        expect(requests.length).toBe(1);
        const stringifiedRequest = await (0, utils_1.stringifyRequest)(requests[0]);
        const request = JSON.parse(stringifiedRequest);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(request.response.status).toBe(502);
        expect(request.postData).toMatchObject({ foo: 'bar' });
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(request.response.statusText).toBe('Bad Gateway');
    });
});
