"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = tslib_1.__importStar(require("path"));
const fs_1 = require("fs");
const simple_fake_server_1 = require("simple-fake-server");
const SUT = tslib_1.__importStar(require("../../playwright-fluent"));
const utils_1 = require("../../../utils");
describe('Playwright Fluent - onRequestTo(url).respondWith()', () => {
    let p;
    let fakeServer = undefined;
    beforeAll(() => {
        fakeServer = new simple_fake_server_1.FakeServer(1242);
        fakeServer.start();
        //The FakeServer now listens on http://localhost:1242
    });
    afterAll(() => {
        if (fakeServer) {
            fakeServer.stop();
        }
    });
    beforeEach(() => {
        p = new SUT.PlaywrightFluent();
    });
    afterEach(async () => {
        await p.close();
    });
    test('should intercept only POST requests to a rest API', async () => {
        // Given
        const htmlContent = (0, fs_1.readFileSync)(`${path.join(__dirname, 'on-request-to-respond-with.post.test.html')}`);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/app')
                .willReturn(htmlContent.toString(), 200);
        const responseBody = {
            prop1: 'foobar',
        };
        const responseHeaders = {
            'foo-header': 'bar',
        };
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .post()
                .to('/foobar')
                .willReturn(responseBody, 200, responseHeaders);
        // When
        const mockResponseBody = { prop1: 'mocked-prop1', prop2: 'mocked-prop2' };
        await p
            .withBrowser('chromium')
            .withOptions({ headless: true })
            .withCursor()
            .recordRequestsTo('/foobar')
            .onRequestTo('/foobar', { method: 'POST' })
            .respondWith({
            status: 200,
            headers: responseHeaders,
            body: mockResponseBody,
        })
            .navigateTo('http://localhost:1242/app');
        // Then requests to /foobar should be intercepted
        // And response should be mocked
        await p.waitForStabilityOf(async () => p.getRecordedRequestsTo('/foobar').length, {
            stabilityInMilliseconds: 5000,
        });
        const foobarRequests = p.getRecordedRequestsTo('/foobar');
        expect(Array.isArray(foobarRequests)).toBe(true);
        expect(foobarRequests.length).toBe(2);
        const stringifiedSentRequest = await (0, utils_1.stringifyRequest)(foobarRequests[1]);
        const sentRequest = JSON.parse(stringifiedSentRequest);
        expect(sentRequest.url).toContain('?foo=bar');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.status).toBe(200);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.headers['foo-header']).toBe(responseHeaders['foo-header']);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.payload).toMatchObject(mockResponseBody);
    });
    test('should intercept only POST requests to a rest API - functional version', async () => {
        // Given
        const htmlContent = (0, fs_1.readFileSync)(`${path.join(__dirname, 'on-request-to-respond-with.post.test.html')}`);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/app')
                .willReturn(htmlContent.toString(), 200);
        const responseBody = {
            prop1: 'foobar',
        };
        const responseHeaders = {
            'foo-header': 'bar',
        };
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .post()
                .to('/foobar')
                .willReturn(responseBody, 200, responseHeaders);
        // When
        const mockResponseBody = { prop1: 'mocked-prop1', prop2: 'mocked-prop2' };
        // const harFile = path.join(__dirname, 'on-request-to-respond-with.har');
        await p
            .withBrowser('chromium')
            .withOptions({ headless: true })
            .withCursor()
            //.recordNetworkActivity({ path: harFile })
            .recordRequestsTo('/foobar')
            .onRequestTo('/foobar', { method: 'POST' })
            .respondWith((request) => {
            const url = request.url();
            // eslint-disable-next-line no-console
            console.log(url);
            return {
                status: 200,
                headers: responseHeaders,
                body: mockResponseBody,
            };
        })
            .navigateTo('http://localhost:1242/app');
        // Then requests to /foobar should be intercepted
        // And response should be mocked
        await p.waitForStabilityOf(async () => p.getRecordedRequestsTo('/foobar').length, {
            stabilityInMilliseconds: 5000,
        });
        const foobarRequests = p.getRecordedRequestsTo('/foobar');
        expect(Array.isArray(foobarRequests)).toBe(true);
        expect(foobarRequests.length).toBe(2);
        const stringifiedSentRequest = await (0, utils_1.stringifyRequest)(foobarRequests[1]);
        const sentRequest = JSON.parse(stringifiedSentRequest);
        expect(sentRequest.url).toContain('?foo=bar');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.status).toBe(200);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.headers['foo-header']).toBe(responseHeaders['foo-header']);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.payload).toMatchObject(mockResponseBody);
    });
    test('should mock reponse with data coming from a HAR file', async () => {
        // Given
        const htmlContent = (0, fs_1.readFileSync)(`${path.join(__dirname, 'on-request-to-respond-with.post.test.html')}`);
        const harFile = path.join(__dirname, 'on-request-to-respond-with.har');
        const harData = (0, utils_1.getHarDataFrom)(harFile);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/app')
                .willReturn(htmlContent.toString(), 200);
        const responseBody = {
            prop1: 'foobar',
        };
        const responseHeaders = {
            'foo-header': 'bar',
        };
        const mockResponseBody = { prop1: 'mocked-prop1', prop2: 'mocked-prop2' };
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .post()
                .to('/foobar')
                .willReturn(responseBody, 200, responseHeaders);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/foobar')
                .willReturn(responseBody, 200, responseHeaders);
        // When
        await p
            .withBrowser('chromium')
            .withOptions({ headless: true })
            .withCursor()
            .recordRequestsTo('/foobar')
            .onRequestTo('/foobar', { method: 'POST' })
            .respondWith((request) => {
            const harResponse = (0, utils_1.getHarResponseFor)(request, harData);
            const response = (0, utils_1.getHarResponseContentAs)(harResponse);
            return {
                status: harResponse === null || harResponse === void 0 ? void 0 : harResponse.status,
                headers: (0, utils_1.harHeadersToHttpHeaders)(harResponse === null || harResponse === void 0 ? void 0 : harResponse.headers),
                body: response,
            };
        })
            .navigateTo('http://localhost:1242/app');
        // Then requests to /foobar should be intercepted
        // And response should be mocked
        await p.waitForStabilityOf(async () => p.getRecordedRequestsTo('/foobar').length, {
            stabilityInMilliseconds: 5000,
        });
        const foobarRequests = p.getRecordedRequestsTo('/foobar');
        expect(Array.isArray(foobarRequests)).toBe(true);
        expect(foobarRequests.length).toBe(2);
        const stringifiedSentRequest = await (0, utils_1.stringifyRequest)(foobarRequests[1]);
        const sentRequest = JSON.parse(stringifiedSentRequest);
        expect(sentRequest.url).toContain('?foo=bar');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.status).toBe(200);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.headers['foo-header']).toBe(responseHeaders['foo-header']);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.payload).toMatchObject(mockResponseBody);
    });
});
