"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = tslib_1.__importStar(require("path"));
const fs_1 = require("fs");
const simple_fake_server_1 = require("simple-fake-server");
const SUT = tslib_1.__importStar(require("../../playwright-fluent"));
const utils_1 = require("../../../utils");
describe('Playwright Fluent - onRequestTo(url).respondWith()', () => {
    let p;
    let fakeServer = undefined;
    beforeAll(() => {
        fakeServer = new simple_fake_server_1.FakeServer(1243);
        fakeServer.start();
        //The FakeServer now listens on http://localhost:1243
    });
    afterAll(() => {
        if (fakeServer) {
            fakeServer.stop();
        }
    });
    beforeEach(() => {
        p = new SUT.PlaywrightFluent();
    });
    afterEach(async () => {
        await p.close();
    });
    test('should intercept GET requests to a rest API', async () => {
        // Given
        const htmlContent = (0, fs_1.readFileSync)(`${path.join(__dirname, 'on-request-to-respond-with.test.html')}`);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/app')
                .willReturn(htmlContent.toString(), 200);
        const responseBody = {
            prop1: 'foobar',
        };
        const responseBodyBaz = {
            prop1: 'foobaz',
        };
        const responseHeaders = {
            'foo-header': 'bar',
        };
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/foobar')
                .willReturn(responseBody, 200, responseHeaders);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/yo')
                .willReturn(responseBodyBaz, 200, responseHeaders);
        // When
        const mockResponseBody = { prop1: 'mocked-prop1', prop2: 'mocked-prop2' };
        await p
            .withBrowser('chromium')
            .withOptions({ headless: true })
            .withCursor()
            .recordRequestsTo('/foobar')
            .onRequestTo('/foobar')
            .respondWith({
            status: 200,
            headers: responseHeaders,
            body: mockResponseBody,
        })
            .navigateTo('http://localhost:1243/app');
        // Then requests to /foobar should be intercepted
        // And response should be mocked
        await p.waitForStabilityOf(async () => p.getRecordedRequestsTo('/foobar').length, {
            stabilityInMilliseconds: 5000,
        });
        const foobarRequests = p.getRecordedRequestsTo('/foobar');
        expect(Array.isArray(foobarRequests)).toBe(true);
        expect(foobarRequests.length).toBe(1);
        const stringifiedSentRequest = await (0, utils_1.stringifyRequest)(foobarRequests[0]);
        const sentRequest = JSON.parse(stringifiedSentRequest);
        expect(sentRequest.url).toContain('?foo=bar');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.status).toBe(200);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.headers['foo-header']).toBe(responseHeaders['foo-header']);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.payload).toMatchObject(mockResponseBody);
    });
    test('should intercept GET requests to a rest API -functional version', async () => {
        // Given
        const htmlContent = (0, fs_1.readFileSync)(`${path.join(__dirname, 'on-request-to-respond-with.test.html')}`);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/app')
                .willReturn(htmlContent.toString(), 200);
        const responseBody = {
            prop1: 'foobar',
        };
        const responseBodyBaz = {
            prop1: 'foobaz',
        };
        const responseHeaders = {
            'foo-header': 'bar',
        };
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/foobar')
                .willReturn(responseBody, 200, responseHeaders);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/yo')
                .willReturn(responseBodyBaz, 200, responseHeaders);
        // When
        const mockResponseBody = { prop1: 'mocked-prop1', prop2: 'mocked-prop2' };
        await p
            .withBrowser('chromium')
            .withOptions({ headless: true })
            .withCursor()
            .recordRequestsTo('/foobar')
            .onRequestTo('/foobar')
            .respondWith((request) => {
            const url = request.url();
            if (url.includes('?foo=bar')) {
                return {
                    status: 200,
                    headers: responseHeaders,
                    body: mockResponseBody,
                };
            }
            throw new Error(`Cannot handle to request '${url}'`);
        })
            .navigateTo('http://localhost:1243/app');
        // Then requests to /foobar should be intercepted
        // And response should be mocked
        await p.waitForStabilityOf(async () => p.getRecordedRequestsTo('/foobar').length, {
            stabilityInMilliseconds: 5000,
        });
        const foobarRequests = p.getRecordedRequestsTo('/foobar');
        expect(Array.isArray(foobarRequests)).toBe(true);
        expect(foobarRequests.length).toBe(1);
        const stringifiedSentRequest = await (0, utils_1.stringifyRequest)(foobarRequests[0]);
        const sentRequest = JSON.parse(stringifiedSentRequest);
        expect(sentRequest.url).toContain('?foo=bar');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.status).toBe(200);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.headers['foo-header']).toBe(responseHeaders['foo-header']);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.payload).toMatchObject(mockResponseBody);
    });
    test('should intercept GET requests to a rest API by responding with HTTP Status 401', async () => {
        // Given
        const htmlContent = (0, fs_1.readFileSync)(`${path.join(__dirname, 'on-request-to-respond-with.test.html')}`);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/app')
                .willReturn(htmlContent.toString(), 200);
        const responseBody = {
            prop1: 'foobar',
        };
        const responseBodyBaz = {
            prop1: 'foobaz',
        };
        const responseHeaders = {
            'foo-header': 'bar',
        };
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/foobar')
                .willReturn(responseBody, 200, responseHeaders);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/yo')
                .willReturn(responseBodyBaz, 200, responseHeaders);
        // When
        await p
            .withBrowser('chromium')
            .withOptions({ headless: true })
            .withCursor()
            .recordRequestsTo('/foobar')
            .onRequestTo('/foobar')
            .respondWith({
            status: 401,
            body: 'sorry, you have no access',
        })
            .navigateTo('http://localhost:1243/app');
        // Then requests to /foobar should be intercepted
        // And response should be mocked
        await p.waitForStabilityOf(async () => p.getRecordedRequestsTo('/foobar').length, {
            stabilityInMilliseconds: 5000,
        });
        const foobarRequests = p.getRecordedRequestsTo('/foobar');
        expect(Array.isArray(foobarRequests)).toBe(true);
        expect(foobarRequests.length).toBe(1);
        const stringifiedSentRequest = await (0, utils_1.stringifyRequest)(foobarRequests[0]);
        const sentRequest = JSON.parse(stringifiedSentRequest);
        expect(sentRequest.url).toContain('?foo=bar');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.status).toBe(401);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.headers['content-type']).toBe('text/plain');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.headers['access-control-allow-origin']).toBe('*');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.headers['access-control-allow-credentials']).toBe('true');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.payload).toBe('sorry, you have no access');
    });
});
