"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = tslib_1.__importStar(require("path"));
const fs_1 = require("fs");
const simple_fake_server_1 = require("simple-fake-server");
const SUT = tslib_1.__importStar(require("../../playwright-fluent"));
const utils_1 = require("../../../utils");
describe('Playwright Fluent - withMocks()', () => {
    let p;
    let fakeServer = undefined;
    beforeAll(() => {
        fakeServer = new simple_fake_server_1.FakeServer(1248);
        fakeServer.start();
        //The FakeServer now listens on http://localhost:1248
    });
    afterAll(() => {
        if (fakeServer) {
            fakeServer.stop();
        }
    });
    beforeEach(() => {
        p = new SUT.PlaywrightFluent();
    });
    afterEach(async () => {
        await p.close();
    });
    test('should intercept GET requests to a rest API', async () => {
        // Given
        const htmlContent = (0, fs_1.readFileSync)(`${path.join(__dirname, 'with-mocks.test.html')}`);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/app')
                .willReturn(htmlContent.toString(), 200);
        const responseBody = {
            prop1: 'foobar',
        };
        const responseBodyBaz = {
            prop1: 'foobaz',
        };
        const responseHeaders = {
            'foo-header': 'bar',
        };
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/foobar')
                .willReturn(responseBody, 200, responseHeaders);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/yo')
                .willReturn(responseBodyBaz, 200, responseHeaders);
        // When
        const mockedResponseBody = { prop1: 'mocked-prop1', prop2: 'mocked-prop2' };
        const mock1 = {
            displayName: 'mock GET /foobar requests',
            urlMatcher: (url) => url.includes('/foobar'),
            methodMatcher: (method) => method === 'GET',
            responseType: 'json',
            jsonResponse: () => mockedResponseBody,
            enrichResponseHeaders: (headers) => {
                return {
                    ...headers,
                    ...responseHeaders,
                };
            },
        };
        const mocks = [mock1];
        await p
            .withBrowser('chromium')
            .withOptions({ headless: true })
            .withCursor()
            .recordRequestsTo('/foobar')
            .withMocks(mocks)
            .navigateTo('http://localhost:1248/app');
        // Then requests to /foobar should be intercepted
        // And response should be mocked
        await p.waitForStabilityOf(async () => p.getRecordedRequestsTo('/foobar').length, {
            stabilityInMilliseconds: 5000,
        });
        const foobarRequests = p.getRecordedRequestsTo('/foobar');
        expect(Array.isArray(foobarRequests)).toBe(true);
        expect(foobarRequests.length).toBe(1);
        const stringifiedSentRequest = await (0, utils_1.stringifyRequest)(foobarRequests[0]);
        const sentRequest = JSON.parse(stringifiedSentRequest);
        expect(sentRequest.url).toContain('?foo=bar');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.status).toBe(200);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.headers['foo-header']).toBe(responseHeaders['foo-header']);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.payload).toMatchObject(mockedResponseBody);
        // Given mock is registered
        expect(p.hasMockWithDisplayName(mock1.displayName)).toBe(true);
        // When I remove the mock
        p.removeMocksWithDisplayName(mock1.displayName);
        // Then mock is not registered any more
        expect(p.hasMockWithDisplayName(mock1.displayName)).toBe(false);
    });
    test('should intercept GET requests to a rest API only for the second request', async () => {
        // Given
        const htmlContent = (0, fs_1.readFileSync)(`${path.join(__dirname, 'with-mocks.test.html')}`);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/app')
                .willReturn(htmlContent.toString(), 200);
        const responseBody = {
            prop1: 'foobar',
        };
        const responseBodyBaz = {
            prop1: 'foobaz',
        };
        const responseHeaders = {
            'foo-header': 'bar',
        };
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/foobar')
                .willReturn(responseBody, 200, responseHeaders);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/yo')
                .willReturn(responseBodyBaz, 200, responseHeaders);
        let numberOfCall = 0;
        const mock1 = {
            displayName: 'GET /yo?foo=baz1 requests - simulate throttling',
            urlMatcher: (url) => url.includes('/yo'),
            methodMatcher: (method) => method === 'GET',
            queryStringMatcher: (q) => q.foo === 'baz1',
            contextMatcher: (ctx) => {
                numberOfCall += 1;
                const context = ctx;
                if (typeof context.callIndex !== 'number') {
                    context.callIndex = 0;
                }
                context.callIndex += 1;
                return context.callIndex === 2;
            },
            responseType: 'string',
            rawResponse: () => 'sorry, tow many requests',
            status: 429,
        };
        const mocks = [mock1];
        // When
        await p
            .withBrowser('chromium')
            .withOptions({ headless: true })
            .withCursor()
            .recordRequestsTo('/yo')
            .withMocks(mocks)
            .navigateTo('http://localhost:1248/app');
        // Then requests to /foobar should be intercepted
        // And response should be mocked
        await p.waitUntil(async () => p.getRecordedRequestsTo('/yo').length >= 3, {
            stabilityInMilliseconds: 1000,
        });
        const yoRequests = p.getRecordedRequestsTo('yo?foo=baz1');
        expect(Array.isArray(yoRequests)).toBe(true);
        expect(yoRequests.length).toBe(3);
        expect(numberOfCall).toBe(3);
        const firstRequest = await (0, utils_1.toRequestInfo)(yoRequests[0]);
        const secondRequest = await (0, utils_1.toRequestInfo)(yoRequests[1]);
        const thirdRequest = await (0, utils_1.toRequestInfo)(yoRequests[2]);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(firstRequest.response.status).toBe(200);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(secondRequest.response.status).toBe(429);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(thirdRequest.response.status).toBe(200);
    });
    test('should intercept GET requests to a rest API by responding with HTTP Status 401 with a 10s delay', async () => {
        // Given
        const htmlContent = (0, fs_1.readFileSync)(`${path.join(__dirname, 'with-mocks.test.html')}`);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/app')
                .willReturn(htmlContent.toString(), 200);
        const responseBody = {
            prop1: 'foobar',
        };
        const responseBodyBaz = {
            prop1: 'foobaz',
        };
        const responseHeaders = {
            'foo-header': 'bar',
        };
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/foobar')
                .willReturn(responseBody, 200, responseHeaders);
        fakeServer &&
            // prettier-ignore
            fakeServer.http
                .get()
                .to('/yo')
                .willReturn(responseBodyBaz, 200, responseHeaders);
        const mock1 = {
            displayName: 'return HTTP 401 on GET /foobar requests',
            urlMatcher: (url) => url.includes('/foobar'),
            methodMatcher: (method) => method === 'GET',
            responseType: 'string',
            rawResponse: () => 'sorry, you have no access',
            status: 401,
            delayInMilliseconds: 10000,
        };
        const mocks = [mock1];
        // When
        const startTime = new Date();
        await p
            .withBrowser('chromium')
            .withOptions({ headless: true })
            .withCursor()
            .recordRequestsTo('/foobar')
            .withMocks(mocks)
            .navigateTo('http://localhost:1248/app');
        // Then requests to /foobar should be intercepted
        // And response should be mocked
        await p.waitUntil(async () => p.getRecordedRequestsTo('/foobar').length >= 1, {
            stabilityInMilliseconds: 1000,
        });
        const endTime = new Date();
        const elapsedInMilliseconds = endTime.getTime() - startTime.getTime();
        expect(Math.abs(elapsedInMilliseconds)).toBeGreaterThanOrEqual(10000);
        const foobarRequests = p.getRecordedRequestsTo('/foobar');
        expect(Array.isArray(foobarRequests)).toBe(true);
        expect(foobarRequests.length).toBe(1);
        const stringifiedSentRequest = await (0, utils_1.stringifyRequest)(foobarRequests[0]);
        const sentRequest = JSON.parse(stringifiedSentRequest);
        expect(sentRequest.url).toContain('?foo=bar');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.status).toBe(401);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.headers['content-type']).toBe('text/plain');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.headers['access-control-allow-origin']).toBe('*');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.headers['access-control-allow-credentials']).toBe('true');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        expect(sentRequest.response.payload).toBe('sorry, you have no access');
    });
});
